package org.mikgul;

import java.io.IOException;
import java.security.InvalidParameterException;
import java.util.Map;
import java.util.Scanner;

public class Main {
    public static void main(String[] args) throws IOException {
        WeatherJsonParser parser = new WeatherJsonParser();
        Map<String, Coordinates> cities;
        try {
            cities = parser.prepareCities();
        } catch (IOException e) {
            System.out.println("Blad przy wczytywaniu danych z `cities.json`: " + e);
            return;
        }
        Scanner scanner = new Scanner(System.in);

        while (true) {
            System.out.print("Podaj nazwe miasta (q - wyjscie):\n>> ");
            String cityName = scanner.nextLine();

            // uzytkownik chce wyjsc z programu
            if ("q".equalsIgnoreCase(cityName)) {
                if (WeatherEntry.noEntries()) break;

                System.out.print("Podaj format do zapisu - pdf/json/xml (q - wyjscie bez zapisu):\n>> ");
                String format = scanner.nextLine().toLowerCase();
                try {
                    switch (format) {
                        case "json":
                            WeatherEntry.saveToJson();
                            System.out.println("Zapisano w JSON.");
                            break;
                        case "xml":
                            WeatherEntry.saveToXml();
                            System.out.println("Zapisano w XML.");
                            break;
                        case "pdf":
                            WeatherEntry.saveToPdf();
                            System.out.println("Zapisano w PDF.");
                            break;
                        case "q":
                            System.out.println("Nie zapisano w zadnym pliku.");
                            break;
                        default:
                            System.out.println("Nieprawidlowy format.");
                            continue;
                    }
                } catch (IOException e) {
                    System.out.println("Blad podczas zapisu do pliku: " + e);
                }
                break;
            }

            // uzytkownik chce otrzymac dane o podanym miescie
            Coordinates coordinates = cities.get(cityName);
            if (coordinates != null) {
                WeatherResponse res = new WeatherResponse();
                WeatherEntry entry;
                try {
                     entry = parser.parseWeatherData(cityName, res.sendWeatherRequest(coordinates));
                } catch(InvalidParameterException e) {
                    System.out.println("Parsowanie zapytania nie powiodlo sie: " + e + "\n");
                    continue;
                }
                System.out.println(entry);
            } else {
                System.out.println("Podane miasto nie jest obslugiwane (sprawdz plik `cities.json`).\n");
            }
        }
    }
}