package org.mikgul;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDPageContentStream;
import org.apache.pdfbox.pdmodel.font.PDFont;
import org.apache.pdfbox.pdmodel.font.PDType0Font;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

public class WeatherEntry {
    private static final List<WeatherEntry> allEntries = new ArrayList<>();
    private final String miasto;
    private final String pogoda;
    private final double temperatura;
    private final int cisnienie;
    private final int wilgotnosc;

    public WeatherEntry(String city, String description, double temperature, int pressure, int humidity) {
        this.miasto = city;
        this.pogoda = description;
        this.temperatura = temperature;
        this.cisnienie = pressure;
        this.wilgotnosc = humidity;
        allEntries.add(this);
    }

    public static void saveToJson() throws IOException {
        File jsonFile = new File(generateFileName(".json"));
        ObjectMapper mapper = new ObjectMapper();
        mapper.writeValue(jsonFile, allEntries);
    }

    public static void saveToXml() throws IOException {
        File xmlFile = new File(generateFileName(".xml"));
        XmlMapper mapper = new XmlMapper();
        mapper.writeValue(xmlFile, allEntries);
    }

    public static void saveToPdf() throws IOException {
        // zapis do txt
        String txtFile = "src/weather-results/temp/temp.txt";

        Path parentFolderPath = Paths.get("src/weather-results/temp");
        File folder = parentFolderPath.toFile();
        if (!folder.exists()) {
            try {
                Files.createDirectories(parentFolderPath);
            } catch (IOException e) {
                System.out.println("Brak folderu `src/weather-results/temp`. Tworzenie folderu nie powiodlo sie.");
            }
        }

        try (FileWriter writer = new FileWriter(txtFile)) {
            for (WeatherEntry entry : allEntries) {
                writer.write(entry.toString() + System.lineSeparator());
            }
        }

        // konwersja na pdf
        PDDocument document = new PDDocument();
        PDPage page = new PDPage();

        document.addPage(page);

        try (PDPageContentStream contentStream = new PDPageContentStream(document, page)) {
            List<String> lines = Files.readAllLines(Paths.get(txtFile));

            PDFont font = PDType0Font.load(document, new File("AbhayaLibre-Regular.ttf"));
            int yPosition = 700;
            for (String line : lines) {
                contentStream.beginText();
                contentStream.setFont(font, 14);
                contentStream.newLineAtOffset(100, yPosition);
                contentStream.showText(line);
                contentStream.endText();
                yPosition -= 20;
            }

        }
        document.save(generateFileName(".pdf"));
        document.close();
    }

    private static String generateFileName(String format) {
        String directoryPath = "src/weather-results/";
        String baseFileName = "weather-session-";

        File directory = new File(directoryPath);

        int i = 1;
        while (true) {
            String newFileName = baseFileName + i + format;

            File newFile = new File(directory, newFileName);
            if (!newFile.exists()) {
                return directoryPath + newFileName;
            }
            i++;
        }
    }

    public String toString() {
        return String.format("Pogoda w %s:\n%s, temperatura: %.2f C; ciśnienie: %d hPa; wilgotność: %d%%\n",
                miasto, pogoda, temperatura, cisnienie, wilgotnosc);
    }

    public static boolean noEntries() {
        return allEntries.isEmpty();
    }

    public String getMiasto() {
        return miasto;
    }

    public String getPogoda() {
        return pogoda;
    }

    public double getTemperatura() {
        return temperatura;
    }

    public int getCisnienie() {
        return cisnienie;
    }

    public int getWilgotnosc() {
        return wilgotnosc;
    }
}
