package org.mikgul;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.io.File;
import java.io.IOException;
import java.security.InvalidParameterException;
import java.util.HashMap;
import java.util.Map;
import java.util.Spliterator;
import java.util.Spliterators;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;


public class WeatherJsonParser {
    public WeatherJsonParser() {}

    public WeatherEntry parseWeatherData(String cityName, WeatherResponse response) throws IOException {
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode rootNode = objectMapper.readTree(response.getBody());

        if (response.getStatusCode() != 200){
            throw new InvalidParameterException(
                    rootNode.get("cod").asText()
                    + " - "
                    + rootNode.get("message").asText()
            );
        }

        String desc = rootNode.get("weather").get(0).get("description").asText();
        double temp = rootNode.get("main").get("temp").asDouble();
        int press = rootNode.get("main").get("pressure").asInt();
        int hum = rootNode.get("main").get("humidity").asInt();

        return new WeatherEntry(cityName, desc, temp, press, hum);
    }

    public Map<String, Coordinates> prepareCities() throws IOException {
        File jsonFile = new File("cities.json");
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode rootNode = objectMapper.readTree(jsonFile);

        return StreamSupport.stream(Spliterators.spliteratorUnknownSize(rootNode.fields(), Spliterator.ORDERED), false)
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        entry -> new Coordinates(
                                entry.getValue().get("lat").asDouble(),
                                entry.getValue().get("lon").asDouble()
                        ),
                        (existing, replacement) -> existing,
                        HashMap::new
                ));
    }
}
